package com.asgteach.bpmonitor.views;

import com.gluonhq.charm.glisten.animation.BounceInRightTransition;
import com.gluonhq.charm.glisten.application.MobileApplication;
import com.gluonhq.charm.glisten.control.AppBar;
import com.gluonhq.charm.glisten.mvc.View;
import com.gluonhq.charm.glisten.visual.MaterialDesignIcon;
import com.asgteach.bpmonitor.BPMonitor;
import static com.asgteach.bpmonitor.BPMonitor.STAT_VIEW;
import com.asgteach.bpmonitor.model.BPData;
import com.asgteach.bpmonitor.model.Model;
import com.asgteach.bpmonitor.service.Service;
import static com.gluonhq.charm.glisten.application.MobileApplication.HOME_VIEW;
import com.gluonhq.charm.glisten.control.Avatar;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Comparator;
import com.gluonhq.charm.down.Platform;
import javafx.beans.binding.Bindings;
import javafx.beans.binding.StringBinding;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.collections.ListChangeListener;
import javafx.collections.ObservableList;
import javafx.collections.transformation.FilteredList;
import javafx.collections.transformation.SortedList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.chart.LineChart;
import javafx.scene.chart.XYChart;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.Tooltip;
import javafx.scene.image.Image;
import javafx.scene.input.MouseEvent;
import javafx.scene.input.ScrollEvent;
import javafx.scene.input.SwipeEvent;
import javafx.scene.layout.StackPane;
import javax.inject.Inject;

public class GraphPresenter {

    private Button chartButton;
    private Button homeButton;
    private Button statButton;

    private final ChangeListener<Number> bpChangeListener = (obs, ov, nv) -> update();

    // Date and Time formatters
    private final DateTimeFormatter formatter
            = DateTimeFormatter.ofPattern("MMM-dd-yy hh:mma");
    private final DateTimeFormatter dateformatter = DateTimeFormatter.ofPattern("MMM dd yyyy");

    // Line Chart Series
    private final XYChart.Series<String, Number> systolicSeries
            = new XYChart.Series<>();
    private final XYChart.Series<String, Number> diastolicSeries
            = new XYChart.Series<>();
    private final XYChart.Series<String, Number> pulseSeries
            = new XYChart.Series<>();

    // Configure the Line Chart Series observable lists 
    private final ObservableList<XYChart.Data<String, Number>> sysData
            = FXCollections.observableArrayList();

    private final ObservableList<XYChart.Data<String, Number>> diasData
            = FXCollections.observableArrayList();

    private final ObservableList<XYChart.Data<String, Number>> pulseData
            = FXCollections.observableArrayList();

    // Make the chart lists sorted.
    // First, use our custom comparator to convert 
    // LocalDateTime to and from a String
    private final XDateComparator xDateComparator = new XDateComparator();
    // Then define each sorted list to use this customized comparator
    private final SortedList<XYChart.Data<String, Number>> sortedsysData
            = new SortedList<>(sysData, (data1, data2)
                    -> xDateComparator.compare(data1.getXValue(), data2.getXValue()));
    private final SortedList<XYChart.Data<String, Number>> sorteddiasData
            = new SortedList<>(diasData, (data1, data2)
                    -> xDateComparator.compare(data1.getXValue(), data2.getXValue()));
    private final SortedList<XYChart.Data<String, Number>> sortedpulseData
            = new SortedList<>(pulseData, (data1, data2)
                    -> xDateComparator.compare(data1.getXValue(), data2.getXValue()));
    @FXML
    private Label nodeDisplay;
    @FXML
    private StackPane container;
    @FXML
    private Label rangeLabel;

    @Inject
    private Model model;

    private int numReadings = Model.MAXDISPLAY;
    private int maxReadings;

    private StringBinding dateStringBinding;

    // Custom comparator that compares String representations of LocalDateTime
    final class XDateComparator implements Comparator<String> {

        @Override
        public int compare(String t1, String t2) {
            LocalDateTime d1 = LocalDateTime.parse(t1, formatter);
            LocalDateTime d2 = LocalDateTime.parse(t2, formatter);
            return d2.compareTo(d1);
        }
    }

    @FXML
    private View view;
    @FXML
    private Button firstButton;
    @FXML
    private Button prevButton;
    @FXML
    private Button nextButton;
    @FXML
    private Button lastButton;
    @FXML
    private LineChart<String, Number> chart;
    private FilteredList<BPData> filteredList;
    private SortedList<BPData> sortedList;

    @Inject
    private Service service;

    public void initialize() {
        view.setShowTransitionFactory(BounceInRightTransition::new);

        view.showingProperty().addListener(new ChangeListener<Boolean>() {
            @Override
            public void changed(ObservableValue<? extends Boolean> obs, Boolean oldValue, Boolean newValue) {
                if (newValue) {
                    AppBar appBar = MobileApplication.getInstance().getAppBar();
                    appBar.setNavIcon(new Avatar(17, new Image(BPMonitor.class.getResourceAsStream("/icon.png"))));
                    appBar.setTitleText("Chart Data");
                    
                    appBar.getActionItems().add(homeButton = MaterialDesignIcon.HOME.button(e
                            -> MobileApplication.getInstance().switchView(HOME_VIEW)));
                    appBar.getActionItems().add(chartButton = MaterialDesignIcon.TIMELINE.button(e
                            -> System.out.println("Graph Button")));
                    appBar.getActionItems().add(statButton = MaterialDesignIcon.VIEW_WEEK.button(e
                            -> MobileApplication.getInstance().switchView(STAT_VIEW)));
                    
                    homeButton.setTooltip(new Tooltip("VIEW DATA"));
                    chartButton.setTooltip(new Tooltip("VIEW LINE CHART"));
                    statButton.setTooltip(new Tooltip("VIEW STATS"));
                    chartButton.setDisable(true);
                    prevButton.setOnAction((t) -> goPrevious(t));
                    nextButton.setOnAction((t) -> goNext(t));
                    lastButton.setOnAction((t) -> goLast(t));
                    displayChart();
                }
            }
        });

        switch (Platform.getCurrent()) {
            case IOS:
                System.out.println("IOS Device");
                break;
            case ANDROID:
                System.out.println("Android device");
                break;
            case DESKTOP:
                System.out.println("Desktop");
                break;
            default:
                System.out.println("Unknown system.!");
                break;
        }
        // Set up Line Chart
        // Note that our code assumes the default, chart.createSymbols(true).
        // Otherwise, the series data points have no associated nodes
        // and we assume that the nodes exist
        systolicSeries.setName("Systolic");
        chart.getData().add(systolicSeries);
        diastolicSeries.setName("Diastolic");
        chart.getData().add(diastolicSeries);
        pulseSeries.setName("Pulse");
        chart.getData().add(pulseSeries);

        // allow vertical scroll to accommodate orientation change
        container.setOnScroll((ScrollEvent event) -> {
            if (!event.isInertia()) {
                container.setTranslateY(container.getTranslateY() + event.getDeltaY());
            }

            event.consume();
        });

        container.setOnScrollStarted((ScrollEvent event) -> {
            System.out.println("Container: Scroll started event");
            event.consume();
        });

        container.setOnScrollFinished((ScrollEvent event) -> {
            System.out.println("Container: Scroll finished event");
            event.consume();
        });

        // Process swipe events for touch-enabled devices 
        container.setOnSwipeRight((SwipeEvent event) -> {
            System.out.println("container: Swipe right event");
            goPrevious();
            event.consume();
        });

        container.setOnSwipeLeft((SwipeEvent event) -> {
            System.out.println("container: Swipe left event");
            goNext();
            event.consume();
        });
        nodeDisplay.addEventHandler(MouseEvent.MOUSE_CLICKED, event -> {
            nodeDisplay.setText("");
            event.consume();
        });

        // set the icons for the buttons
        firstButton.setGraphic(MaterialDesignIcon.FAST_REWIND.graphic());
        prevButton.setGraphic(MaterialDesignIcon.ARROW_BACK.graphic());
        nextButton.setGraphic(MaterialDesignIcon.ARROW_FORWARD.graphic());
        lastButton.setGraphic(MaterialDesignIcon.FAST_FORWARD.graphic());

        model.getStartIndexProperty().addListener(observable -> {
            System.out.println("start index change");
            filteredList.setPredicate(data -> inRange(sortedList.indexOf(data)));
        });

    }

    private void displayChart() {
        nodeDisplay.setText("");
        // clear chart data
        sysData.clear();
        diasData.clear();
        pulseData.clear();
        System.out.println("Display Chart");
        System.out.println("Service List Property listener: list size = " + service.bpListProperty().get().size());
        sortedList = new SortedList(service.bpListProperty().get());
        // sort so the most recent date is first
        sortedList.setComparator((d1, d2) -> d2.getDate().compareTo(d1.getDate()));
        maxReadings = sortedList.size();
        numReadings = (maxReadings < Model.MAXDISPLAY ? maxReadings : Model.MAXDISPLAY);
        filteredList = new FilteredList(sortedList);
        filteredList.setPredicate(p -> inRange(sortedList.indexOf(p)));

        firstButton.disableProperty().bind(model.getStartIndexProperty().isEqualTo(0));
        prevButton.disableProperty().bind(firstButton.disableProperty());
        lastButton.disableProperty().bind(model.getStartIndexProperty().greaterThanOrEqualTo(Bindings.size(sortedList).subtract(numReadings)));
        nextButton.disableProperty().bind(lastButton.disableProperty());

        // Sets the charts series data to the sorted lists
        // so we don't have to worry about keeping the list sorted!
        systolicSeries.setData(sortedsysData);
        diastolicSeries.setData(sorteddiasData);
        pulseSeries.setData(sortedpulseData);

        for (BPData data : filteredList) {
            addBPDataToChart(data);
            System.out.println("Adding date to chart: " + data.getDate().format(formatter));
        }

        // Add ListChangeListener to the filteredList to keep the Line Chart
        // synchronized with index numbers
        filteredList.addListener(
                (ListChangeListener.Change<? extends BPData> change) -> {
                    while (change.next()) {
                        if (change.wasUpdated()) {
                            System.out.println("Updated in Filtered List ");
                            for (int i = change.getFrom(); i < change.getTo(); ++i) {
                                System.out.println("Updated: " + i + " " + filteredList.get(i));
                            }
                        }
                        if (change.wasRemoved()) {
                            System.out.println("Remove from Filtered List ("
                                    + change.getRemovedSize() + " items)");
                            for (BPData data : change.getRemoved()) {
                                removeBPDataFromChart(change.getFrom());
                            }
                        }
                        if (change.wasAdded()) {
                            System.out.println("Add to Filtered List ("
                                    + change.getAddedSize()
                                    + " items) at index " + change.getFrom());
                            for (BPData data : change.getAddedSubList()) {
                                addBPDataToChart(data);
                            }
                        }
                    }
                });

        dateStringBinding = Bindings.createStringBinding(() -> {
            if (filteredList.size() <= 0) {
                return "";
            }
            return filteredList.get(0).getDate().format(dateformatter)
                    + " -> "
                    + filteredList.get(filteredList.size() - 1).getDate().format(dateformatter);
        }, model.getStartIndexProperty());
        rangeLabel.textProperty().bind(dateStringBinding);

        System.out.println("startIndex = " + model.getStartIndexProperty().get());
        System.out.println("maxReadings = " + maxReadings);
        System.out.println("numReadings = " + numReadings);

    }

    // Add BPData to Line Chart series.
    // All three series share the same XValue value (the date) 
    // for each data point.
    // Create a new XYChart.Data<String, Number> object,
    // add it to the series (unsorted) data,
    // and configure the node for the listeners.
    // Add the data to the unsorted wrapped list
    // and the SortedList wrapper keeps the Line Chart data sorted.
    // This method is called from the filteredList ListChangeListener
    private void addBPDataToChart(BPData data) {
        String myDate = data.getDate().format(formatter);
        // Systolic Series
        XYChart.Data<String, Number> xyData
                = new XYChart.Data<>(myDate, data.getSystolic());
        sysData.add(xyData);
        // This adds the MOUSE event listeners to the data's node
        // (the same listener for each series)
        addChartNodeHandler(xyData);
        // Diastolic Series
        xyData = new XYChart.Data<>(myDate, data.getDiastolic());
        diasData.add(xyData);
        addChartNodeHandler(xyData);
        // Pulse Series
        xyData = new XYChart.Data<>(myDate, data.getPulse());
        pulseData.add(xyData);
        addChartNodeHandler(xyData);
    }

    private void addChartNodeHandler(XYChart.Data<String, Number> data) {
        Node node = data.getNode();
        node.addEventHandler(MouseEvent.MOUSE_ENTERED, event -> {
            nodeDisplay.setText(String.format("%s", data.getYValue()));
            event.consume();
        });
    }

    // This method is called from the filteredList ListChangeListener
    // when one or more BPData elements are removed
    private void removeBPDataFromChart(int index) {
        // Note: we must remove the index associated with the sorted lists.
        // But we can't remove directly from the sorted lists, so we
        // get the correct item and use that as an argument for removal!
        // There is a bug in the charts that throws an exception if the last
        // item in the data observable list is removed. The work-around
        // is to turn off animation for remove operations
        if (index >= 0 && index < sysData.size()) {
            chart.setAnimated(false);
            sysData.remove(sortedsysData.get(index));
            diasData.remove(sorteddiasData.get(index));
            pulseData.remove(sortedpulseData.get(index));
            chart.setAnimated(true);
        }
    }

    @FXML
    private void goFirst(ActionEvent event) {
        System.out.println("Go first");
        model.setStartIndex(0);
    }

    @FXML
    private void goPrevious(ActionEvent event) {
        goPrevious();

    }

    private void goPrevious() {
        System.out.println("Go Previous");
        int i = model.getStartIndex() - numReadings;
        if (i < 0) {
            i = 0;
        }
        model.setStartIndex(i);

    }

    @FXML
    private void goNext(ActionEvent event) {
        goNext();
    }

    private void goNext() {
        System.out.println("Go Next");
        int i = model.getStartIndex() + numReadings;
        if (i >= maxReadings) {
            i = maxReadings - numReadings;
        }
        if (i < 0) {
            i = 0;
        }
        System.out.println("startIndex = " + i
                + ", maxReadings - numReadings =  "
                + (maxReadings - numReadings));
        model.setStartIndex(i);

    }

    @FXML
    private void goLast(ActionEvent event) {
        System.out.println("go Last");
        int i = maxReadings - numReadings;
        if (i < 0) {
            i = 0;
        }
        System.out.println("startIndex = " + i
                + ", maxReadings - numReadings =  "
                + (maxReadings - numReadings));
        model.setStartIndex(i);
    }

    // This is the FilteredList predicate--all based on the startDate control.
    // We convert LocalDateTime to LocalDate--want to ignore time here
    private boolean inRange(int index) {
        return (index >= model.getStartIndex() && index < model.getStartIndex() + numReadings);
    }

    private void update() {

    }
}
