package com.asgteach.message.views;

import com.asgteach.message.cloud.GluonClientProvider;
import com.gluonhq.charm.glisten.application.MobileApplication;
import com.gluonhq.charm.glisten.control.AppBar;
import com.gluonhq.charm.glisten.mvc.View;
import com.gluonhq.charm.glisten.visual.MaterialDesignIcon;
import com.asgteach.message.model.Message;
import com.gluonhq.cloudlink.client.data.DataClient;
import com.gluonhq.cloudlink.client.data.SyncFlag;
import com.gluonhq.connect.ConnectState;
import com.gluonhq.connect.GluonObservableObject;
import com.gluonhq.connect.provider.DataProvider;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.TextArea;

public class PrimaryPresenter {

    private static final String CLOUD_OBJECT_ID = "message";
    private GluonObservableObject<Message> gluonMessage;
    private DataClient dataClient;

    private Message myMessage;

    @FXML
    private View primary;

    @FXML
    private TextArea messageText;
    @FXML
    private Button submit;
    @FXML
    private Button cancel;
    @FXML
    private Label currentMessage;

    public void initialize() {
        primary.showingProperty().addListener((obs, oldValue, newValue) -> {
            if (newValue) {
                AppBar appBar = MobileApplication.getInstance().getAppBar();
                appBar.setNavIcon(MaterialDesignIcon.MENU.button(e
                        -> MobileApplication.getInstance().showLayer(
                                com.asgteach.message.CloudMessage.MENU_LAYER)));
                appBar.setTitleText("Broadcast Message");
            }
        });
        
        dataClient = GluonClientProvider.getDataClient();
        gluonMessage = DataProvider.retrieveObject(dataClient.createObjectDataReader(
                        CLOUD_OBJECT_ID, Message.class,
                        SyncFlag.OBJECT_READ_THROUGH,
                        SyncFlag.OBJECT_WRITE_THROUGH
                ));   

        gluonMessage.stateProperty().addListener((obs, ov, nv) -> {
            if (ConnectState.SUCCEEDED.equals(nv)) {
                System.out.println("connect state is " + nv);
                if (gluonMessage.get() == null) {
                    // object not yet stored, 
                    // initialize it now with a new object and store it
                    System.out.println("Not initialized.");
                    myMessage = new Message("This is a new message.");
                    gluonMessage = DataProvider.storeObject(myMessage, dataClient.createObjectDataWriter(
                            CLOUD_OBJECT_ID, Message.class,
                            SyncFlag.OBJECT_READ_THROUGH,
                            SyncFlag.OBJECT_WRITE_THROUGH
                    ));
                } else {
                    // object already stored previously
                    myMessage = gluonMessage.get();
                    System.out.println("Stored message: " + myMessage.getText());
                }
                currentMessage.textProperty().bind(gluonMessage.get().textProperty());

            } else {
                System.out.println("connect state is " + nv);
            }
        });
        submit.disableProperty().bind(messageText.textProperty().isEmpty());
        cancel.disableProperty().bind(submit.disableProperty());
        primary.requestFocus();
    }

    @FXML
    private void onSubmit(ActionEvent event) {
        System.out.println("Submit for " + messageText.getText());
        gluonMessage.get().setText(messageText.getText());
        messageText.setText("");
    }

    @FXML
    private void onCancel(ActionEvent event) {
        System.out.println("Cancel");
        messageText.setText("");
    }

}
